﻿//This C# code file was released under the Ms-PL license
//http://www.opensource.org/licenses/ms-pl.html
//This script was originally intended for use with Microsoft Dynamics AX
//and maintained and distributed as a project on CodePlex
//http://dynamicsaxadmin.codeplex.com

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Activities;
using Microsoft.TeamFoundation.Build.Client;
using Microsoft.TeamFoundation.Build.Workflow.Activities;
using System.IO;
using System.Text.RegularExpressions;

namespace CodeCrib.AX.TFS
{
    class AutoRunLogOutput
    {
        protected static void OutputLog(CodeActivityContext context, List<KeyValuePair<string, string>> log, bool outputAllAsInfo = false)
        {
            if (log != null)
            {
                foreach (var message in log)
                {
                    if (outputAllAsInfo)
                    {
                        context.TrackBuildMessage(message.Value);
                    }
                    else
                    {
                        switch (message.Key)
                        {
                            case "Info":
                                context.TrackBuildMessage(message.Value);
                                break;
                            case "Warning":
                                context.TrackBuildWarning(message.Value);
                                break;
                            case "Error":
                                context.TrackBuildError(message.Value);
                                break;
                        }
                    }
                }
            }
        }

        public static void Output(CodeActivityContext context, CodeCrib.AX.Client.AutoRun.AxaptaAutoRun autoRun, bool outputAllAsInfo = false, bool skipInfoLog = false)
        {
            autoRun.ParseLog();

            //OutputLog(context, autoRun.Log, outputAllAsInfo);
            
            foreach (var step in autoRun.Steps)
            {
                OutputLog(context, step.Log, outputAllAsInfo);
            }

            if (!skipInfoLog && !string.IsNullOrEmpty(autoRun.InfoLog))
            {
                var lines = autoRun.InfoLog.Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries).ToList();
                foreach (string line in lines)
                {
                    context.TrackBuildMessage(line.Trim());
                }
            }
        }
    }

    [BuildActivity(HostEnvironmentOption.Agent)]
    public class ImportXPO : CodeActivity
    {
        public InArgument<string> ConfigurationFile { get; set; }

        public InArgument<int> TimeOutMinutes { get; set; }

        [RequiredArgument]
        public InArgument<string> XPOFile { get; set; }
        [RequiredArgument]
        public InArgument<StringList> LayerCodes { get; set; }
        [RequiredArgument]
        public InArgument<string> ModelManifestFile { get; set; }

        protected override void Execute(CodeActivityContext context)
        {
            string configurationFile = ConfigurationFile.Get(context);
            int timeoutMinutes = TimeOutMinutes.Get(context);
            string xpoFile = XPOFile.Get(context);
            StringList layerCodes = LayerCodes.Get(context);
            string modelManifest = ModelManifestFile.Get(context);
            string modelName;
            string publisher;
            string layer;
            string layerCode;

            Helper.ExtractClientLayerModelInfo(configurationFile, layerCodes, modelManifest, out modelName, out publisher, out layer, out layerCode);

            var clientConfig = Helper.GetClientConfig(configurationFile);

            CodeCrib.AX.Client.AutoRun.AxaptaAutoRun autoRun = new Client.AutoRun.AxaptaAutoRun() { ExitWhenDone = true, LogFile = string.Format(@"{0}\ImportLog-{1}.xml", System.Environment.ExpandEnvironmentVariables(clientConfig.LogDirectory), System.Guid.NewGuid()) };
            autoRun.Steps.Add(new CodeCrib.AX.Client.AutoRun.XpoImport() { File = xpoFile });

            string autoRunFile = string.Format(@"{0}\AutoRun-ImportXPO-{1}.xml", System.Environment.GetEnvironmentVariable("temp"), Guid.NewGuid());
            CodeCrib.AX.Client.AutoRun.AxaptaAutoRun.SerializeAutoRun(autoRun, autoRunFile);

            context.TrackBuildMessage(string.Format("Importing XPO {0} into model {1}", xpoFile, modelName));
            CodeCrib.AX.Client.Client.ExecuteCommand(new CodeCrib.AX.Client.Commands.AutoRun() { ConfigurationFile = configurationFile, Layer = layer, LayerCode = layerCode, Model = modelName, ModelPublisher = publisher, Filename = autoRunFile }, timeoutMinutes);

            AutoRunLogOutput.Output(context, autoRun, true);

            if (System.IO.File.Exists(autoRun.LogFile))
                System.IO.File.Delete(autoRun.LogFile);
            System.IO.File.Delete(autoRunFile);
        }
    }

    [BuildActivity(HostEnvironmentOption.Agent)]
    public class Synchronize : CodeActivity
    {
        public InArgument<string> ConfigurationFile { get; set; }

        public InArgument<int> TimeOutMinutes { get; set; }

        protected override void Execute(CodeActivityContext context)
        {
            int timeoutMinutes = TimeOutMinutes.Get(context);
            string configurationFile = ConfigurationFile.Get(context);

            var clientConfig = Helper.GetClientConfig(configurationFile);

            CodeCrib.AX.Client.AutoRun.AxaptaAutoRun autoRun = new Client.AutoRun.AxaptaAutoRun() { ExitWhenDone = true, LogFile = string.Format(@"{0}\SynchronizeLog-{1}.xml", System.Environment.ExpandEnvironmentVariables(clientConfig.LogDirectory), System.Guid.NewGuid()) };
            autoRun.Steps.Add(new CodeCrib.AX.Client.AutoRun.Synchronize() { SyncDB = true, SyncRoles = true });

            string autoRunFile = string.Format(@"{0}\AutoRun-Synchronize-{1}.xml", System.Environment.GetEnvironmentVariable("temp"), Guid.NewGuid());
            CodeCrib.AX.Client.AutoRun.AxaptaAutoRun.SerializeAutoRun(autoRun, autoRunFile);

            CodeCrib.AX.Client.Client.ExecuteCommand(new CodeCrib.AX.Client.Commands.AutoRun() { ConfigurationFile = configurationFile, Filename = autoRunFile }, timeoutMinutes);

            AutoRunLogOutput.Output(context, autoRun);

            if (System.IO.File.Exists(autoRun.LogFile))
                System.IO.File.Delete(autoRun.LogFile);
            System.IO.File.Delete(autoRunFile);
        }
    }

    [BuildActivity(HostEnvironmentOption.Agent)]
    public class ImportLabels : CodeActivity
    {
        public InArgument<string> ConfigurationFile { get; set; }

        [RequiredArgument]
        public InArgument<string> LabelFilesFolder { get; set; }
        [RequiredArgument]
        public InArgument<StringList> LayerCodes { get; set; }
        [RequiredArgument]
        public InArgument<string> ModelManifestFile { get; set; }

        public InArgument<int> TimeOutMinutes { get; set; }

        public static bool IsEmptyLabelFile(string labelFile)
        {
            bool isEmptyFile = true;

            if (File.Exists(labelFile))
            {
                using (StreamReader streamReader = new StreamReader(File.OpenRead(labelFile)))
                {
                    int lineCounter = 0;
                    while (isEmptyFile && !streamReader.EndOfStream && lineCounter < 50)
                    {
                        string line = streamReader.ReadLine().Trim();

                        Match match = Regex.Match(line, @"@.{3}\d+\s.+");
                        if (match.Success)
                        {
                            isEmptyFile = false;
                        }

                        lineCounter++;
                    }
                }
            }

            return isEmptyFile;
        }

        protected override void Execute(CodeActivityContext context)
        {
            int timeoutMinutes = TimeOutMinutes.Get(context);
            string configurationFile = ConfigurationFile.Get(context);
            StringList layerCodes = LayerCodes.Get(context);
            string labelFilesFolder = LabelFilesFolder.Get(context);
            
            if (!Directory.Exists(labelFilesFolder))
            {
                context.TrackBuildWarning(string.Format("Label file folder {0} not found.", labelFilesFolder));
                return;
            }

            string modelName;
            string publisher;
            string layer;
            string layerCode;
            string modelManifest = ModelManifestFile.Get(context);

            Helper.ExtractClientLayerModelInfo(configurationFile, layerCodes, modelManifest, out modelName, out publisher, out layer, out layerCode);
            
            var clientConfig = Helper.GetClientConfig(configurationFile);
            CodeCrib.AX.Client.AutoRun.AxaptaAutoRun autoRun = new Client.AutoRun.AxaptaAutoRun() { ExitWhenDone = true, LogFile = string.Format(@"{0}\LabelFlushLog-{1}.xml", System.Environment.ExpandEnvironmentVariables(clientConfig.LogDirectory), System.Guid.NewGuid()) };
            CodeCrib.AX.Client.Commands.ImportLabelFile importCommand = new CodeCrib.AX.Client.Commands.ImportLabelFile() { ConfigurationFile = configurationFile, Layer = layer, LayerCode = layerCode, Model = modelName, ModelPublisher = publisher };

            foreach (string filename in Directory.GetFiles(labelFilesFolder, "*.ald"))
            {
                if (!IsEmptyLabelFile(filename))
                {
                    context.TrackBuildMessage(string.Format("Importing label file {0} into model {1} ({2})", filename, modelName, publisher));
                    importCommand.Filename = filename;
                    CodeCrib.AX.Client.Client.ExecuteCommand(importCommand, timeoutMinutes);

                    string labelFile = System.IO.Path.GetFileNameWithoutExtension(filename).Substring(2, 3);
                    string labelLanguage = System.IO.Path.GetFileNameWithoutExtension(filename).Substring(5);

                    autoRun.Steps.Add(new CodeCrib.AX.Client.AutoRun.Run() { Type = Client.AutoRun.RunType.@class, Name = "Global", Method = "info", Parameters = string.Format("strFmt(\"Flush label {0} language {1}: %1\", Label::flush(\"{0}\",\"{1}\"))", labelFile, labelLanguage) });
                }
            }

            string autoRunFile = string.Format(@"{0}\AutoRun-LabelFlush-{1}.xml", System.Environment.GetEnvironmentVariable("temp"), Guid.NewGuid());
            CodeCrib.AX.Client.AutoRun.AxaptaAutoRun.SerializeAutoRun(autoRun, autoRunFile);
            context.TrackBuildMessage(string.Format("Flushing imported label files"));
            CodeCrib.AX.Client.Client.ExecuteCommand(new CodeCrib.AX.Client.Commands.AutoRun() { ConfigurationFile = configurationFile, Layer = layer, LayerCode = layerCode, Model = modelName, ModelPublisher = publisher, Filename = autoRunFile }, timeoutMinutes);

            AutoRunLogOutput.Output(context, autoRun, true);

            if (System.IO.File.Exists(autoRun.LogFile))
                System.IO.File.Delete(autoRun.LogFile);
            System.IO.File.Delete(autoRunFile);
        }
    }

    [BuildActivity(HostEnvironmentOption.Agent)]
    public class ImportVSProject : CodeActivity
    {
        public InArgument<string> ConfigurationFile { get; set; }

        public InArgument<int> TimeOutMinutes { get; set; }

        [RequiredArgument]
        public InArgument<string> VSProjectsFolder { get; set; }
        [RequiredArgument]
        public InArgument<StringList> LayerCodes { get; set; }
        [RequiredArgument]
        public InArgument<string> ModelManifestFile { get; set; }

        protected override void Execute(CodeActivityContext context)
        {
            string configurationFile = ConfigurationFile.Get(context);
            int timeoutMinutes = TimeOutMinutes.Get(context);
            string vsProjectsFolder = VSProjectsFolder.Get(context);
            StringList layerCodes = LayerCodes.Get(context);

            if (!Directory.Exists(vsProjectsFolder))
            {
                context.TrackBuildMessage(string.Format("VS Projects folder {0} not found.", vsProjectsFolder));
                return;
            }

            string modelName;
            string publisher;
            string layer;
            string layerCode;
            string modelManifest = ModelManifestFile.Get(context);

            Helper.ExtractClientLayerModelInfo(configurationFile, layerCodes,  modelManifest, out modelName, out publisher, out layer, out layerCode);

            var clientConfig = Helper.GetClientConfig(configurationFile);


            CodeCrib.AX.Client.AutoRun.AxaptaAutoRun autoRun = new Client.AutoRun.AxaptaAutoRun() { ExitWhenDone = true, LogFile = string.Format(@"{0}\VSImportLog-{1}.xml", System.Environment.ExpandEnvironmentVariables(clientConfig.LogDirectory), System.Guid.NewGuid()) };

            var filesToProcess = from filter in new[] { "*.csproj", "*.dynamicsproj", "*.vbproj" }
                                 select Directory.GetFiles(vsProjectsFolder, filter, SearchOption.AllDirectories);

            foreach (string filename in filesToProcess.SelectMany(f => f))
            {
                autoRun.Steps.Add(new CodeCrib.AX.Client.AutoRun.Run() { Type = Client.AutoRun.RunType.@class, Name = "SysTreeNodeVSProject", Method = "importProject", Parameters = string.Format("@'{0}'", filename) });
            }

            string autoRunFile = string.Format(@"{0}\AutoRun-VSImport-{1}.xml", System.Environment.GetEnvironmentVariable("temp"), Guid.NewGuid());
            CodeCrib.AX.Client.AutoRun.AxaptaAutoRun.SerializeAutoRun(autoRun, autoRunFile);

            context.TrackBuildMessage(string.Format("Importing VS Projects from folder {0} into model {1}", vsProjectsFolder, modelName));
            CodeCrib.AX.Client.Client.ExecuteCommand(new CodeCrib.AX.Client.Commands.AutoRun() { ConfigurationFile = configurationFile, Layer = layer, LayerCode = layerCode, Model = modelName, ModelPublisher = publisher, Filename = autoRunFile }, timeoutMinutes);

            AutoRunLogOutput.Output(context, autoRun, true);

            if (System.IO.File.Exists(autoRun.LogFile))
                System.IO.File.Delete(autoRun.LogFile);
            System.IO.File.Delete(autoRunFile);
        }
    }
/*
    private void CreateVSProjectImportXML(string sourcesFolder, bool recursive, string xmlFilename, bool append, string logFile, CodeActivityContext context)
        {
            if (System.IO.Directory.Exists(sourcesFolder))
            {
                AutoRun.AxaptaAutoRun autoRun = null;
                if (append)
                {
                    autoRun = AutoRun.AxaptaAutoRun.FindOrCreate(xmlFilename);
                }
                else
                {
                    autoRun = new AutoRun.AxaptaAutoRun();
                }

                autoRun.ExitWhenDone = true;
                if (logFile != "")
                {
                    autoRun.LogFile = logFile;
                }
                autoRun.Version = "4.0";
                autoRun.Steps = new List<AutoRun.AutorunElement>();

                List<string> files = new List<string>();

                GetProjectFiles(sourcesFolder, recursive, files, "*.csproj");

                if (files.Count != 0)
                {
                    List<string>.Enumerator fileEnumerator = files.GetEnumerator();
                    // Add import steps
                    while (fileEnumerator.MoveNext())
                    {
                        autoRun.Steps.Add(new AutoRun.Run() { type = AutoRun.RunType.@class, name = "SysTreeNodeVSProject", method = "importProject", parameters = string.Format("@'{0}'", fileEnumerator.Current) });
                    }
                    fileEnumerator = files.GetEnumerator();
                    // Add compile steps
                    while (fileEnumerator.MoveNext())
                    {
                        autoRun.Steps.Add(new AutoRun.CompileApplication() { node = string.Format(@"\Visual Studio Projects\C Sharp Projects\{0}", System.IO.Path.GetFileNameWithoutExtension(fileEnumerator.Current)), crossReference = false });
                    }

                    AutoRun.AxaptaAutoRun.SerializeAutoRun(autoRun, xmlFilename);
                }
                else
                {
                    context.TrackBuildMessage("No Visual Studio Projects found.");
                    // do not create autorun.xml file if there's nothing in it
                }
            }
            else
            {
                context.TrackBuildMessage("No Visual Studio Projects found.");
            }
        }
*/
}
