﻿//This C# code file was released under the Ms-PL license
//http://www.opensource.org/licenses/ms-pl.html
//This script was originally intended for use with Microsoft Dynamics AX
//and maintained and distributed as a project on CodePlex
//http://dynamicsaxadmin.codeplex.com

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;

using System.Activities;
using Microsoft.TeamFoundation.Build.Client;
using Microsoft.TeamFoundation.Build.Workflow.Activities;

namespace CodeCrib.AX.TFS
{
    [BuildActivity(HostEnvironmentOption.Agent)]
    public class Compile : CodeActivity
    {
        public InArgument<string> ClientExecutablePath { get; set; }

        public InArgument<bool> UpdateCrossReference { get; set; }
        public InArgument<int> TimeOutMinutes { get; set; }

        public InArgument<string> ConfigurationFile { get; set; }

        public InArgument<StringList> LayerCodes { get; set; }
        public InArgument<string> ModelManifestFile { get; set; }


        protected override void Execute(CodeActivityContext context)
        {
            bool updateXRef = UpdateCrossReference.Get(context);
            int timeOutMinutes = TimeOutMinutes.Get(context);
            string clientExePath = ClientExecutablePath.Get(context);
            string configurationFile = ConfigurationFile.Get(context);

            context.TrackBuildMessage(string.Format("Compiling application {0} cross-reference update.", updateXRef ? "with" : "without"));

            CodeCrib.AX.Client.Commands.Compile compile = new CodeCrib.AX.Client.Commands.Compile()
                {
                    Minimize = true,
                    LazyClassLoading = true,
                    LazyTableLoading = true,
                    UpdateCrossReference = updateXRef
                };

            if (!string.IsNullOrEmpty(configurationFile))
            {
                compile.ConfigurationFile = configurationFile;
            }

            StringList layerCodes = LayerCodes.Get(context);
            if (layerCodes != null)
            {
                string modelManifest = ModelManifestFile.Get(context);
                if (!String.IsNullOrEmpty(modelManifest))
                {
                    string model;
                    string publisher;
                    string layer;
                    string layerCode;

                    Helper.ExtractClientLayerModelInfo(configurationFile, layerCodes, modelManifest, out model, out publisher, out layer, out layerCode);

                    compile.Model = model;
                    compile.ModelPublisher = publisher;
                    compile.Layer = layer;
                    compile.LayerCode = layerCode;
                }
            }

            if (string.IsNullOrEmpty(clientExePath))
                CodeCrib.AX.Client.Client.ExecuteCommand(compile, timeOutMinutes);
            else
                CodeCrib.AX.Client.Client.ExecuteCommand(clientExePath, compile, timeOutMinutes);

            var clientConfig = Helper.GetClientConfig(configurationFile);

            CodeCrib.AX.Client.CompileOutput output = null;

            try
            {
                output = CodeCrib.AX.Client.CompileOutput.CreateFromFile(string.Format(@"{0}\{1}", System.Environment.ExpandEnvironmentVariables(clientConfig.LogDirectory), "AxCompileAll.html"));
            }
            catch (System.IO.FileNotFoundException)
            {
                throw new Exception("Compile log could not be found");
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("Error parsing compile log: {0}", ex.Message));
            }

            bool hasErrors = false;
            foreach (var item in output.Output)
            {
                string compileMessage = String.Format("{0}, line {1}, column {2} : {3}", item.TreeNodePath, item.LineNumber, item.ColumnNumber, item.Message);
                switch (item.Severity)
                {
                    // Compile Errors
                    case 0:
                        context.TrackBuildError(compileMessage);
                        hasErrors = true;
                        break;
                    // Compile Warnings
                    case 1:
                    case 2:
                    case 3:
                        context.TrackBuildWarning(compileMessage);
                        break;
                    // Best practices
                    case 4:
                        context.TrackBuildMessage(string.Format("BP: {0}", compileMessage), BuildMessageImportance.Low);
                        break;
                    // TODOs
                    case 254:
                    case 255:
                        context.TrackBuildWarning(string.Format("TODO: {0}", compileMessage));
                        break;
                    // "Other"
                    default:
                        context.TrackBuildMessage(compileMessage);
                        break;
                }
            }
            if (hasErrors)
                throw new Exception("Compile error(s) found");
        }
    }
}
